package com.deekor.fingerpainting;

import android.content.Context;
import android.graphics.Color;
import android.graphics.Matrix;
import android.graphics.Paint;
import android.graphics.Path;
import android.graphics.RectF;
import android.os.Bundle;
import android.os.Parcelable;
import android.view.MotionEvent;
import android.view.View;
import android.graphics.Canvas;

import java.util.ArrayList;

/**
 * Created by Dee on 9/16/13.
 */
public class PaintAreaView extends View implements View.OnTouchListener
{
    public ArrayList<CustomStroke> customStrokes;
    public int currentStroke;
    private Context context;
    public boolean watchMode;
    public float percent;

    public PaintAreaView(Context context)
    {
        super(context);
        this.context = context;
        customStrokes = new ArrayList<CustomStroke>();
        this.setOnTouchListener(this);
        customStrokes.add(new CustomStroke(Color.BLACK));
        currentStroke = 0;
        watchMode = false;
        percent = 0;
    }

    public void changeColor(int color)
    {
        customStrokes.add(new CustomStroke(color));
        currentStroke++;
    }

    public int currentColor()
    {
        return customStrokes.get(currentStroke).color;
    }

    @Override
    protected void onDraw(Canvas canvas)
    {
        System.out.println("View info:\n Width: " + getWidth() +"\n  Height: "+ getHeight());
        if(watchMode)//Only show a percentage
        {

            int count = 0;
            //Get the amount of instructions
            for(CustomStroke c : customStrokes)
            {
                count = count + c.instructions.size();
            }
            //Calculate stopping instruction
            int stop = (int)(count * percent);
            System.out.println("Percentage is " + percent + "\nNumber of instructions: "+count+ "\nStopping at: "+ stop);
            int ins = 0;
            for(CustomStroke c : customStrokes)
            {
                    if(ins >= stop)
                        break;
                    //The Paint
                    Paint paint = new Paint();
                    paint.setAntiAlias(true);
                    paint.setStrokeWidth(10f);
                    paint.setColor(c.color);
                    paint.setStyle(Paint.Style.STROKE);
                    paint.setStrokeJoin(Paint.Join.ROUND);

                    //The actual path
                    Path p = new Path();
                    for(Instruction i : c.instructions)
                    {
                        if(i.line)
                        {
                            p.lineTo(i.x, i.y);
                        }
                        else
                        {
                            p.moveTo(i.x, i.y);
                        }
                        ins++; //Count an instruction
                        if(ins >= stop)
                            break;
                    }
                    if(getWidth() > getHeight())
                    {
                        Matrix scaleMatrix = new Matrix();
                        RectF rectF = new RectF();
                        p.computeBounds(rectF, true);
                        scaleMatrix.setScale(1.6f, .6f,0,0);
                        p.transform(scaleMatrix);
                    }
                    canvas.drawPath(p, paint);
            }

        }
        else //Just draw mode.
        {
            for(CustomStroke c : customStrokes)
            {
                    //The Paint
                    Paint paint = new Paint();
                    paint.setAntiAlias(true);
                    paint.setStrokeWidth(10f);
                    paint.setColor(c.color);
                    paint.setStyle(Paint.Style.STROKE);
                    paint.setStrokeJoin(Paint.Join.ROUND);

                    //The actual path
                    Path p = new Path();
                    for(Instruction i : c.instructions)
                    {
                        if(i.line)
                        {
                            p.lineTo(i.x, i.y);
                        }
                        else
                        {
                            p.moveTo(i.x, i.y);
                        }
                    }
                if(getWidth() > getHeight())
                {
                Matrix scaleMatrix = new Matrix();
                RectF rectF = new RectF();
                p.computeBounds(rectF, true);
                scaleMatrix.setScale(1.6f, .6f,0,0);
                p.transform(scaleMatrix);
                }
                canvas.drawPath(p, paint);
            }
        }
        //canvas.rotate(180f,0,0);
        canvas.scale(-300f, -300f);
        System.out.println("Getting called");
            setBackgroundColor(0xFFf7f6eb);
    }


    @Override
    public boolean onTouch(View view, MotionEvent motionEvent) {

        if(!watchMode)
        {
            float x = motionEvent.getX();
            float y = motionEvent.getY();

            switch (motionEvent.getAction()) {
                case MotionEvent.ACTION_DOWN:
                    customStrokes.get(currentStroke).instructions.add(new Instruction(false,x,y));
                    //System.out.println("Down!");
                    return true;
                case MotionEvent.ACTION_MOVE:
                    customStrokes.get(currentStroke).instructions.add(new Instruction(true,x,y));
                    //System.out.println("Move!");
                    break;
                case MotionEvent.ACTION_UP:
                    // nothing to do
                    break;
                default:
                    return false;
            }
            invalidate();
        }
        return true;
    }
}


