package com.deekor.fingerpainting;

import android.content.Intent;
import android.content.SharedPreferences;
import android.graphics.Color;
import android.graphics.Path;
import android.graphics.Point;
import android.os.Bundle;
import android.app.Activity;
import android.view.Display;
import android.view.Menu;
import android.view.View;
import android.view.ViewGroup;
import android.view.Window;
import android.widget.Button;
import android.widget.LinearLayout;
import android.widget.SeekBar;

import com.google.gson.Gson;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.FileWriter;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.util.ArrayList;
import java.util.Timer;
import java.util.TimerTask;


public class PaintCanvas extends Activity {
    PaintAreaView paintCanvas;
    SharedPreferences.OnSharedPreferenceChangeListener listener;
    Button colorButton;
    Button modeButton;
    Button playButton;
    Timer timer;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        requestWindowFeature(Window.FEATURE_NO_TITLE);

        //Layout for the activity
        LinearLayout layout = new LinearLayout(this);
        layout.setOrientation(LinearLayout.VERTICAL);

        //For sizing the views
        Display display = getWindowManager().getDefaultDisplay();
        Point size = new Point();
        display.getSize(size);
        System.out.println("Screen size is: "+ size);
        int height = size.y;

        paintCanvas = new PaintAreaView(this);
        paintCanvas.setId(5);


        //Shared preferences for persistent current color
        SharedPreferences settings = getSharedPreferences("colors", MODE_PRIVATE);
        paintCanvas.changeColor(settings.getInt("currentColor", Color.BLACK));
        //Listener to know when color changes.
         listener = new SharedPreferences.OnSharedPreferenceChangeListener() {
            public void onSharedPreferenceChanged(SharedPreferences settings, String key) {
                if(key.equals("currentColor"))
                {
                    System.out.println("Color Changed to "+ settings.getInt("currentColor", 0));
                    paintCanvas.changeColor(settings.getInt("currentColor", Color.BLACK));
                }
            }
        };
        settings.registerOnSharedPreferenceChangeListener(listener);

        //maintain watch mode
        final SharedPreferences.Editor editor = settings.edit();
        paintCanvas.watchMode = settings.getBoolean("watchMode", false);
        paintCanvas.percent = settings.getFloat("percent", 0);



        //Bottom Menu Setup
        final LinearLayout bot = new LinearLayout(this);
        bot.setId(100);

        //Slider
        final SeekBar slider = new SeekBar(this);
        slider.setOnSeekBarChangeListener(new SeekBar.OnSeekBarChangeListener() {
            @Override
            public void onProgressChanged(SeekBar seekBar, int i, boolean fromUser) {
                paintCanvas.percent = ((float)i / 100);
                editor.putInt("progress", slider.getProgress());
                editor.commit();
                System.out.println("Slider at: " + i);
                paintCanvas.invalidate();
            }

            @Override
            public void onStartTrackingTouch(SeekBar seekBar) {

            }

            @Override
            public void onStopTrackingTouch(SeekBar seekBar) {

            }
        });

        //Play / Pause Button
        playButton = new Button(this);
        playButton.setText("Play");
        playButton.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View view) {
                if(playButton.getText().equals("Play"))
                {
                    playButton.setText("Pause");
                    timer = new Timer();
                    timer.scheduleAtFixedRate(new TimerTask()
                    {
                        @Override
                        public void run()
                        {
                            slider.incrementProgressBy(1);
                        }
                    },0,100);
                }
                else
                {
                    playButton.setText("Play");
                    timer.cancel();
                }
            }

        });


        //Color Button
        colorButton = new Button(this);
        colorButton.setText("Color");
        colorButton.setBackgroundColor(paintCanvas.currentColor());
        if(settings.getInt("currentColor", Color.BLACK) ==  Color.BLACK)
            colorButton.setTextColor(Color.WHITE);
        else
            colorButton.setTextColor(Color.BLACK);
        colorButton.setOnClickListener(new View.OnClickListener()
        {
            public void onClick(View view)
            {
                System.out.println("Color Button Clicked!");
                Intent intent = new Intent(PaintCanvas.this, PaletteActivity.class);
                startActivity(intent);

            }

        });
        bot.addView(colorButton);

        //Mode Button
        modeButton = new Button(this);
        //if/else for persistence in orientation change
        if(paintCanvas.watchMode)
        {
            modeButton.setText("Draw Mode");
            paintCanvas.watchMode = true;
            bot.removeView(colorButton);
            bot.addView(playButton);
            bot.addView(slider,new LinearLayout.LayoutParams(ViewGroup.LayoutParams.MATCH_PARENT, ViewGroup.LayoutParams.WRAP_CONTENT));
            slider.setProgress(settings.getInt("progress", 0));
        }
        else
        {
            modeButton.setText("Watch mode");
        }
        bot.addView(modeButton,0);
        modeButton.setOnClickListener(new View.OnClickListener()
        {
            public void onClick(View view)
            {
                System.out.println("Mode Button Clicked!");
                if(paintCanvas.watchMode)
                {
                    modeButton.setText("Watch Mode");
                    paintCanvas.watchMode = false;
                    bot.addView(colorButton, 0);
                    bot.removeView(slider);
                    bot.removeView(playButton);
                    paintCanvas.invalidate();
                    editor.putBoolean("watchMode", false);
                    editor.commit();
                }
                else
                {
                    modeButton.setText("Draw Mode");
                    paintCanvas.watchMode = true;
                    bot.removeView(colorButton);
                    bot.addView(playButton);
                    bot.addView(slider,new LinearLayout.LayoutParams(ViewGroup.LayoutParams.MATCH_PARENT, ViewGroup.LayoutParams.WRAP_CONTENT));
                    slider.setProgress(0);
                    paintCanvas.percent = 0;
                    paintCanvas.invalidate();
                    editor.putBoolean("watchMode", true);
                    editor.commit();

                }
            }

        });

        //Display everything
        layout.addView(paintCanvas, new LinearLayout.LayoutParams(ViewGroup.LayoutParams.MATCH_PARENT, (int)(height*0.80),0));
        //layout.addView(palette, new LinearLayout.LayoutParams(ViewGroup.LayoutParams.MATCH_PARENT, ViewGroup.LayoutParams.WRAP_CONTENT));
        layout.addView(bot, new LinearLayout.LayoutParams(ViewGroup.LayoutParams.MATCH_PARENT, ViewGroup.LayoutParams.WRAP_CONTENT));




        //Try Load here!
        /**Read Colors**/
        try
        {

            FileInputStream ins = openFileInput("drawing.dat");
            ObjectInputStream reader = new ObjectInputStream(ins);
            paintCanvas.customStrokes = (ArrayList<CustomStroke>)reader.readObject();
            paintCanvas.currentStroke = paintCanvas.customStrokes.size() -1;

            System.out.println("Drawing Loaded");
        }
        catch (Exception e)
        {
            e.printStackTrace();
            System.out.println("Error Thrown in read colors");
        }

        setContentView(layout);
    }



    @Override
    protected void onDestroy()
    {
        super.onDestroy();

        System.out.println("Saving");
        SharedPreferences settings = getSharedPreferences("colors", MODE_PRIVATE);
        settings.unregisterOnSharedPreferenceChangeListener(listener);

        /**Write Drawing**/
        try
        {
            FileOutputStream os = openFileOutput("drawing.dat", MODE_PRIVATE);
            ObjectOutputStream output = new ObjectOutputStream(os);
            output.writeObject(paintCanvas.customStrokes);
            output.close();
        }
        catch (Exception e)
        {
            e.printStackTrace();
            System.out.println("Error Thrown in write colors");
        }

    }

    @Override
    public boolean onCreateOptionsMenu(Menu menu) {
        // Inflate the menu; this adds items to the action bar if it is present.
        getMenuInflater().inflate(R.menu.canvas, menu);
        return true;
    }

    @Override
    public void onResume()
    {
        super.onResume();
        System.out.println("Resumed Main Activity!");
        SharedPreferences settings = getSharedPreferences("colors", MODE_PRIVATE);
        colorButton.setBackgroundColor(settings.getInt("currentColor", Color.BLACK));
        if(settings.getInt("currentColor", Color.BLACK) ==  Color.BLACK)
            colorButton.setTextColor(Color.WHITE);
        else
            colorButton.setTextColor(Color.BLACK);
    }


    
}
