package com.deekor.fingerpainting;

import android.content.Context;
import android.content.SharedPreferences;
import android.graphics.Canvas;
import android.graphics.Color;
import android.graphics.Paint;
import android.graphics.Rect;
import android.graphics.RectF;
import android.os.Bundle;
import android.os.Parcelable;
import android.util.FloatMath;
import android.view.MotionEvent;
import android.view.View;
import android.view.ViewGroup;
import android.widget.Button;

import java.util.ArrayList;

/**
 * Created by Dee on 9/16/13.
 */
public class PaletteView extends ViewGroup {

    private Context context;
    private boolean mixColors;
    private ArrayList<Integer> newColors;



    public PaletteView(Context context)
    {
        super(context);
        mixColors = false;
        this.context = context;
        newColors = new ArrayList<Integer>();
        setBackgroundColor(0xFF9c6e00);

        //Set up the mix button
        Button mix =  new Button(context);
        mix.setText("MIX");
        mix.setTextSize(10);
        addView(mix);

        mix.setOnClickListener(new OnClickListener() {
            @Override
            public void onClick(View view) {
                Button b = (Button)view;
                if(mixColors)
                {
                    mixColors = false;
                    b.setBackgroundResource(android.R.drawable.btn_default);


                }
                else
                {
                    mixColors = true;
                    b.setBackgroundColor(Color.YELLOW);
                }
            }
        });


        //addColor(Color.MAGENTA);
        addColor(Color.BLACK);
        addColor(Color.WHITE);
        addColor(Color.RED);
        addColor(Color.BLUE);
        addColor(Color.GREEN);

    }

    @Override
    protected void onDraw(Canvas canvas)
    {
        RectF oval = new RectF(10,10,getWidth() -10,getHeight()-10);
        Paint ovalColor = new Paint();
        ovalColor.setColor(0xFFccad62);
        canvas.drawOval(oval, ovalColor);
    }

    public void addColor(int color)
    {
        final PaletteActivity palette = (PaletteActivity)context;
        Paint p = new Paint();
        p.setColor(color);
        final PaintView pview = new PaintView(context);
        pview.setPaintColor(color);

        pview.setOnTouchListener(new OnTouchListener() {
            @Override
            public boolean onTouch(View view, MotionEvent motionEvent) {
                //What happens when a paint is clicked,  either mix it
                //or make it the active color
                if(motionEvent.getAction() == MotionEvent.ACTION_DOWN)
                {
                    if(mixColors)
                    {
                       int first = palette.getColor();
                       int second = pview.currentColor();

                       int r = (Color.red(first) + Color.red(second))/2;
                       int g = (Color.green(first) + Color.green(second))/2;
                       int b = (Color.blue(first) + Color.blue(second))/2;

                       addColor(Color.rgb(r,g,b));
                       //add to our list for save as well
                        newColors.add(Color.rgb(r,g,b));
                        palette.saveColors(newColors);


                       onLayout(true,0,0,0,0); //fix the weird circle when adding a color



                    }
                    else
                    {
                        palette.changeColor(pview.currentColor());
                        pview.highlight();
                        for (int i = 1; i < getChildCount(); i++)
                        {
                            PaintView child = (PaintView)getChildAt(i);
                            if(child != pview)
                                child.unHighlight();
                        }
                    }
                }

                return true;
            }
        });

        addView(pview);

    }

    private void removeColor(int color)
    {
        for (int i = 1; i < getChildCount(); i++)
        {
            PaintView p = (PaintView)getChildAt(i);
            if(p.currentColor() == color)
                removeViewAt(i);
        }

    }


    @Override
    protected void onLayout(boolean changed, int left, int top, int right, int bottom)
    {
        View button =  getChildAt(0);
        button.layout(getWidth() / 2 - 50, getHeight() / 2 - 50, getWidth() / 2 + 50, getHeight() / 2 + 50);

        for (int i = 1; i < getChildCount(); i++) {

            //Get the child to layout
            View child = getChildAt(i);

            //Calculate the oval
            float width = 40.0f;
            float height = 40.0f;

            float angle = (float) (2.0 * Math.PI) * ((float)i / (float)getChildCount());
            float a = getWidth() / 2 - (getWidth() * 0.08f);
            float b = getHeight() /2 - (getHeight() * 0.08f);
            float c = getWidth() * 0.455f;
            float d = getHeight() * 0.43f;

            child.setX(c + (a * FloatMath.cos(angle)));
            child.setY(d + (b * FloatMath.sin(angle)));

            int l,t,r,bot;

            l = (int)(child.getX() - (width /2));
            t = (int)(child.getY() - (height /2));
            r =  (int)(child.getX() + (width /2));
            bot = (int)(child.getY() + (height /2));

            //Place the child
            child.layout(l,t,r,bot);

        }
    }

    @Override
    protected Parcelable onSaveInstanceState()
    {
        Bundle bundle = new Bundle();
        bundle.putParcelable("super", super.onSaveInstanceState());
        ArrayList<View> children = new ArrayList<View>();
        for (int i = 1; i < getChildCount(); i++)
        {
            children.add(getChildAt(i));
        }
        bundle.putSerializable("children", children);
        return bundle;
    }

    @Override
    protected void onRestoreInstanceState(Parcelable state)
    {
        if(state instanceof Bundle)
        {
            Bundle bundle = (Bundle)state;
            super.onRestoreInstanceState(bundle.getParcelable("super"));
            ArrayList<View> children = (ArrayList<View>)bundle.getSerializable("children");

            removeAllViews();

            //Set up the mix button
            Button mix =  new Button(context);
            mix.setText("MIX");
            mix.setTextSize(10);
            addView(mix);

            mix.setOnClickListener(new OnClickListener() {
                @Override
                public void onClick(View view) {
                    Button b = (Button)view;
                    if(mixColors)
                    {
                        mixColors = false;
                        b.setBackgroundResource(android.R.drawable.btn_default);


                    }
                    else
                    {
                        mixColors = true;
                        b.setBackgroundColor(Color.YELLOW);
                    }
                }
            });

            for(View c : children)
            {
                PaintView p = (PaintView)c;
                addColor(p.currentColor());
            }
        }
    }
}
